#include "do_menu.h"
#include <stdio.h>
#include "globals.h"
#include "font.h"
#include "hw_md.h"
#include "gfx.h"
#include "program_status.h"

void print_seed(uint16_t plane, uint8_t highlighted_digit, uint16_t x, uint16_t y);
void change_seed_digit(uint8_t digit, int8_t amount);
void manage_seed();

uint8_t do_menu() {
	prev_buttons |= SEGA_CTRL_START;

	uint8_t restart_menu = 0;

	set_palette(font_palette_menu, 0x00, sizeof(font_palette_menu)/sizeof(uint16_t));

	while (1) {
		tile_rectangle(0x0000, PLANE_A, 0, 0, 40, 28);
		tile_rectangle(0x0000, PLANE_B, 0, 0, 40, 28);

		print_string("MAIN MENU", PLANE_A, 15, 2);

		print_string("START GAME", PLANE_A, 12, 7);
		print_string("MANAGE SEED", PLANE_A, 12, 9);
		print_string("EXIT", PLANE_A, 12, 11);

		uint8_t menu_index = 0;
		const char * space = " ";
		const char arrow[2] = { ' ' + 59, '\0' };
		print_string(arrow, PLANE_A, 10, 7 + menu_index*2);
		while (1) {
			buttons = get_pad(0);

			if (!(prev_buttons & SEGA_CTRL_UP) && (buttons & SEGA_CTRL_UP)) {
				print_string(space, PLANE_A, 10, 7 + menu_index*2);
				if (menu_index == 0) {
					menu_index = 2;
				}
				else {
					menu_index--;
				}
				print_string(arrow, PLANE_A, 10, 7 + menu_index*2);
			}
			if (!(prev_buttons & SEGA_CTRL_DOWN) && (buttons & SEGA_CTRL_DOWN)) {
				print_string(space, PLANE_A, 10, 7 + menu_index*2);
				if (menu_index == 2) {
					menu_index = 0;
				}
				else {
					menu_index++;
				}
				print_string(arrow, PLANE_A, 10, 7 + menu_index*2);
			}

			if (!(prev_buttons & SEGA_CTRL_START) && (buttons & SEGA_CTRL_START)) {
				switch (menu_index) {
				case 0:
					return PROGRAM_GAME;
					break;
				case 1:
					manage_seed();
					restart_menu = 1;
					break;
				case 2:
					return PROGRAM_EXIT;
					break;
				default:
					break;
				}
			}

			if (restart_menu) {
				break;
			}

			wait_for_vblank();
			prev_buttons = buttons;
		}
		restart_menu = 0;
	}
}

void print_seed(uint16_t plane, uint8_t highlighted_digit, uint16_t x, uint16_t y) {
	SET_VDP_REG(15, 2);

	ACCESS_VDP(VRAM_W, plane + SCREEN_LOC(x, y));
	for (int8_t i = 7; i >= 0; i--) {
		uint8_t hex_digit = (seed >> (i*4)) & 0xF;
		uint16_t digit_tile;
		if (hex_digit <= 9) {
			digit_tile = '0' + hex_digit;
		}
		else {
			digit_tile = 'A' + (hex_digit - 0xA);
		}
		if (highlighted_digit != 0 && ((highlighted_digit - 1) == i)) {
			digit_tile |= 0x2000;
		}
		DATA_PORT_WORD(digit_tile);
	}
}

void change_seed_digit(uint8_t digit, int8_t amount) {
	int8_t seed_digit = (seed >> (digit * 4)) & 0xF;

	if (amount == 1) {
		if (seed_digit == 15) {
			seed_digit = 0;
		}
		else {
			seed_digit++;
		}
	}
	else if (amount == -1) {
		if (seed_digit == 0) {
			seed_digit = 15;
		}
		else {
			seed_digit--;
		}
	}

	seed = (seed & ~((uint32_t)(0xF << (digit * 4)))) | (((uint32_t)seed_digit) << (digit * 4));
}

void manage_seed() {
	prev_buttons |= SEGA_CTRL_START;

	set_palette(font_palette_seed, 0x10, sizeof(font_palette_seed)/sizeof(uint16_t));

	tile_rectangle(0x0000, PLANE_A, 0, 0, 40, 28);
	tile_rectangle(0x0000, PLANE_B, 0, 0, 40, 28);

	print_string("MANAGE SEED", PLANE_A, 15, 2);

	print_string("SEED", PLANE_A, 12, 7);
	print_seed(PLANE_A, 0, 17, 7);
	print_string("CLEAR SEED", PLANE_A, 12, 9);
	print_string("BACK", PLANE_A, 12, 11);

	uint8_t managing_seed = 1;
	uint8_t menu_index = 0;
	const char * space = " ";
	const char arrow[2] = { ' ' + 59, '\0' };
	print_string(arrow, PLANE_A, 10, 7 + menu_index*2);
	while (managing_seed) {
		buttons = get_pad(0);

		if (!(prev_buttons & SEGA_CTRL_UP) && (buttons & SEGA_CTRL_UP)) {
			print_string(space, PLANE_A, 10, 7 + menu_index*2);
			if (menu_index == 0) {
				menu_index = 2;
			}
			else {
				menu_index--;
			}
			print_string(arrow, PLANE_A, 10, 7 + menu_index*2);
		}
		if (!(prev_buttons & SEGA_CTRL_DOWN) && (buttons & SEGA_CTRL_DOWN)) {
			print_string(space, PLANE_A, 10, 7 + menu_index*2);
			if (menu_index == 2) {
				menu_index = 0;
			}
			else {
				menu_index++;
			}
			print_string(arrow, PLANE_A, 10, 7 + menu_index*2);
		}

		if (!(prev_buttons & SEGA_CTRL_START) && (buttons & SEGA_CTRL_START)) {
			prev_buttons |= SEGA_CTRL_START;
			uint8_t changing_seed;
			uint8_t current_digit;
			switch (menu_index) {
			case 0:
				changing_seed = 1;
				current_digit = 1;
				while (changing_seed) {
					buttons = get_pad(0);

					if (!(prev_buttons & SEGA_CTRL_LEFT) && (buttons & SEGA_CTRL_LEFT)) {
						if (current_digit == 8) {
							current_digit = 1;
						}
						else {
							current_digit++;
						}
					}
					if (!(prev_buttons & SEGA_CTRL_RIGHT) && (buttons & SEGA_CTRL_RIGHT)) {
						if (current_digit == 1) {
							current_digit = 8;
						}
						else {
							current_digit--;
						}
					}

					uint8_t digit = current_digit - 1;
					if (!(prev_buttons & SEGA_CTRL_UP) && (buttons & SEGA_CTRL_UP)) {
						change_seed_digit(digit, 1);
					}
					if (!(prev_buttons & SEGA_CTRL_DOWN) && (buttons & SEGA_CTRL_DOWN)) {
						change_seed_digit(digit, -1);
					}

					if (!(prev_buttons & SEGA_CTRL_START) && (buttons & SEGA_CTRL_START)) {
						changing_seed = 0;
						current_digit = 0;
					}

					print_seed(PLANE_A, current_digit, 17, 7);
					wait_for_vblank();
					prev_buttons = buttons;
				}
				break;
			case 1:
				seed = 0;
				print_seed(PLANE_A, 0, 17, 7);
				print_string(space, PLANE_A, 10, 7 + menu_index*2);
				menu_index = 0;
				print_string(arrow, PLANE_A, 10, 7 + menu_index*2);
				break;
			case 2:
				managing_seed = 0;
				prev_buttons |= SEGA_CTRL_START;
				break;
			}
		}

		wait_for_vblank();
		prev_buttons = buttons;
	}
}
