#pragma once

#include <stdint.h>

#define SEGA_CTRL_BUTTONS   0x0FFF
#define SEGA_CTRL_UP        0x0001
#define SEGA_CTRL_DOWN      0x0002
#define SEGA_CTRL_LEFT      0x0004
#define SEGA_CTRL_RIGHT     0x0008
#define SEGA_CTRL_B         0x0010
#define SEGA_CTRL_C         0x0020
#define SEGA_CTRL_A         0x0040
#define SEGA_CTRL_START     0x0080
#define SEGA_CTRL_Z         0x0100
#define SEGA_CTRL_Y         0x0200
#define SEGA_CTRL_X         0x0400
#define SEGA_CTRL_MODE      0x0800

#define SEGA_CTRL_TYPE      0xF000
#define SEGA_CTRL_THREE     0x0000
#define SEGA_CTRL_SIX       0x1000
#define SEGA_CTRL_NONE      0xF000

#define SCREEN_W 64
#define SCREEN_H 28

#define PLANE_A 0xC000
#define PLANE_W 0xB000
#define PLANE_B 0xE000

#define TILE_LOC(loc) ((loc) << 5)
#define SCREEN_LOC(x, y) (((x) << 1) + (((y) << 1) * SCREEN_W))

#define SET_VDP_REG(reg, val) (*(volatile uint16_t * const)ctrl_port = (0x8000 | ((reg) << 8) | (val)))
#define DATA_PORT_WORD(word) (*(volatile uint16_t * const)data_port = word)
#define DATA_PORT_LONG_WORD(long_word) (*data_port = long_word)

#define VRAM_R 0
#define VRAM_W 1
#define CRAM_R 8
#define CRAM_W 3
#define VSRAM_R 4
#define VSRAM_W 5
#define ADDR_TO_CMD_WORD(addr) (((addr) >> 14) | (((addr) & 0x3FFF) << 16))
#define ACCESS_VDP(code, addr) (*ctrl_port = (((code) & 0xC) << 2) | (((code) & 0x3) << 30) | ADDR_TO_CMD_WORD(addr))

#ifdef __cplusplus
extern "C" {
#endif

extern uint16_t set_sr(uint16_t new_sr);
extern uint16_t get_pad(uint16_t pad);
extern void clear_screen(void);
extern void delay(int32_t count);
extern void wait_for_vblank();
extern void set_palette(const uint16_t * pal, uint32_t start, int32_t count);

extern volatile uint32_t * const data_port;
extern volatile uint32_t * const ctrl_port;

#ifdef __cplusplus
}
#endif
