#include <stdint.h>
#include <string.h>

#include "hw_md.h"
#include "font.h"
#include "gfx.h"
#include "program_status.h"

// Headers that define program status do_* functions.
#include "do_menu.h"
#include "do_game.h"

int main(void) {
	// Tile 0 must always remain all zeros for its pattern, and use palette
	// 0 entry 0 thus being transparent, so that plane A has a transparent
	// tile to use to let the plane B background show through.

	SET_VDP_REG(15, 2);
	ACCESS_VDP(VRAM_W, TILE_LOC(' '));
	for (size_t i = 0; i < sizeof(font_patterns)/sizeof(uint32_t); i++) {
		DATA_PORT_LONG_WORD(font_patterns[i]);
	}

	uint8_t more = 1;
	uint8_t program_status = PROGRAM_MENU;
	while (more) {
		switch (program_status) {
		case PROGRAM_MENU:
			program_status = do_menu();
			break;
		case PROGRAM_GAME:
			program_status = do_game();
			break;
		case PROGRAM_EXIT:
		default:
			more = 0;
			break;
		}
	}

	tile_rectangle(0x0000, PLANE_A, 0, 0, 40, 28);
	tile_rectangle(0x0000, PLANE_B, 0, 0, 40, 28);
	set_palette(font_palette_game, 0x00, sizeof(font_palette_game)/sizeof(uint16_t));
	print_string("EXITED", PLANE_A, 0, 0);

	return 0;
}
