#pragma once

#include <stdint.h>
#include "segarand.h"

/* I may or may not implement the "bugs" in the AC version, or might implement
 * a flag to enable or disable such bugs. The bugs I know of include the row 20
 * bug and music muting if you level-up in a certain way. */

#define INP_NONE	0
#define INP_UP		1
#define INP_RIGHT	2
#define INP_DOWN	4
#define INP_LEFT	8
#define INP_A	16
#define INP_B	32
#define INP_C	64

#define FIELD_HEIGHT	20
#define FIELD_WIDTH	10

/* Constants that game_status uses. */
#define GAME_OVER	0
#define GAME_WAIT	1
#define GAME_ACTIVE	2
#define GAME_CLEAR	3
#define GAME_ARE	4

/* Internally, the AC version orders the pieces IZSOTLJ, I=0 ... J=6. At the
 * time of this writing, there's also the values 7=EMPTY_BLOCK and
 * 8=LINE_CLEARING_BLOCK (shown during line clearing). */
/* pcs[type][dir][axis][block] */
extern const uint8_t pcs[7][4][2][4];

typedef struct {
	uint8_t next;
	uint32_t score;
	uint16_t lines;
	uint8_t level;
	/* 'pc' means "piece," not "personal computer" */
	int16_t pcx;
	int16_t pcy;
	uint8_t pctype;
	uint8_t pcdir;
	uint8_t field[FIELD_HEIGHT][FIELD_WIDTH]; /* At the time of this writing, I don't yet know how the programmer(s) represented the field in the AC tetris, but this will work for now. */
	uint16_t sound; /* In the AC version, only one sound effect played at once. */
	uint16_t music; /* In the AC version, music may or may not be playing alongside the sound effects at various times. */

	/* Generally "internal" state follows. The AC version didn't communicate this stuff directly, visually or aurally. */
	uint32_t rand_state; /* Unsigned exactly-32-bit integer type absolutely required here. */
	uint8_t seq[SEQ_LEN];
	uint16_t seq_pos;
	uint8_t difficulty;
	uint8_t wait_frames;
	uint16_t level_frames;
	uint8_t autoshift_frames;
	uint8_t fall_frames;
	uint8_t slide_frames;
	uint8_t clear_frames;
	uint8_t are_frames;
	uint8_t lines_this_level; /* Turns out level up happens if a total of at least 4 lines is cleared in one level, not just if you clear 4 at once. */
	int8_t line_rows[4]; /* Only valid when game_status == GAME_CLEAR. The score should be displayed in the last index containing a nonnegative value. */
	uint8_t game_status; /* Indicates the global game state. */
	uint16_t prev_input;
} tetris;

void tetris_initialize(tetris * tp, uint8_t difficulty); /* 0=EASY 1=NORMAL 2=HARD 3=HARDEST */
void tetris_do_frame(tetris * tp, uint16_t input); /* "input" is a bitwise-or'd combination of the INP_x constants. */
